﻿Imports System.IO
Imports System.Text
Imports C1.Win.C1FlexGrid

Public Class Form1
    Private Sub FormLoad() Handles Me.Load

        'setup columns
        C1FlexGrid1.Cols(1).Caption = "Name"
        C1FlexGrid1.Cols(2).Caption = "Path"
        C1FlexGrid1.Cols(3).Caption = "Size"
        C1FlexGrid1.Cols(4).Caption = "Creation time"

        'handle events
        C1FlexGrid1.DropMode = DropModeEnum.Manual
        AddHandler C1FlexGrid1.DragDrop, AddressOf FgDragDrop
        AddHandler C1FlexGrid1.DragOver, AddressOf FgDragOver

    End Sub

    Private Sub FgDragOver(sender As Object, e As DragEventArgs)

        'Only allow going further if the file is being dragged either from File explorer Or outlook
        If e.Data.GetData(DataFormats.FileDrop) Is Nothing And e.Data.GetData("FileGroupDescriptor") Is Nothing Then
            Return
        End If

        'Get the info of the cell where the file is being dragged
        Dim info = C1FlexGrid1.HitTest(C1FlexGrid1.PointToClient(New Point(e.X, e.Y)))

        'allow link for only required cell types
        e.Effect = If(info.Type = HitTestTypeEnum.Cell, DragDropEffects.Link, DragDropEffects.None)

    End Sub

    Private Sub FgDragDrop(ByVal sender As Object, ByVal e As DragEventArgs)

        Try
            Dim info = C1FlexGrid1.HitTest(C1FlexGrid1.PointToClient(New Point(e.X, e.Y)))

            'Handling the drag-drop from file explorer similar as before
            If e.Data.GetDataPresent(DataFormats.FileDrop, False) Then
                Dim fileNames As String() = CType(e.Data.GetData(DataFormats.FileDrop), String())

                For Each fileName As String In fileNames
                    If File.Exists(fileName) Then
                        SetRowData(fileName, info.Row)
                    End If
                Next
                C1FlexGrid1.AutoSizeCols()

                'Handling the drag-drop from outlook
            ElseIf e.Data.GetDataPresent("FileGroupDescriptor") Then

                'first step: to get the filename of the attachment and
                'build a full-path name so we can store it in the required folder

                'set up to obtain the FileGroupDescriptor and extract the file name
                Dim theStream As Stream = CType(e.Data.GetData("FileGroupDescriptor"), Stream)
                Dim fileGroupDescriptor As Byte() = New Byte(511) {}
                theStream.Read(fileGroupDescriptor, 0, 512)
                Dim fileName As New StringBuilder("")
                Dim i As Integer = 76

                ' this gets the filename of the passed attached file
                While fileGroupDescriptor(i) <> 0
                    fileName.Append(Convert.ToChar(fileGroupDescriptor(i)))
                    i += 1
                End While
                theStream.Close()

                'Second step:  we have the file name. Now we need to get the actual raw
                'data for the attached file And copy it to disk so we work on it.

                'Build path to store the file, as per your scenario
                Dim theFile = Application.StartupPath & "\" & fileName.ToString()
                Dim ms = CType(e.Data.GetData("FileContents", True), MemoryStream)
                Dim fileBytes = New Byte(ms.Length - 1) {}
                ms.Position = 0
                ms.Read(fileBytes, 0, ms.Length)

                Using fs As New FileStream(theFile, FileMode.Create)
                    fs.Write(fileBytes, 0, fileBytes.Length)
                End Using

                SetRowData(theFile, info.Row)

                C1FlexGrid1.AutoSizeCols()

            End If

        Catch ex As Exception
            Trace.WriteLine("Error in DragDrop function: " & ex.Message)
        End Try
    End Sub

    ''' <summary>
    ''' Sets data of the drag-dropped file into the specfied row
    ''' </summary>
    ''' <param name="fileName">Name of the dropped file</param>
    ''' <param name="droppedRow">Index of row on which file is dropped</param>
    Private Sub SetRowData(fileName As String, droppedRow As Integer)
        Dim infoFile = New FileInfo(fileName)
        C1FlexGrid1(droppedRow, 1) = infoFile.Name
        C1FlexGrid1(droppedRow, 2) = infoFile.FullName
        C1FlexGrid1(droppedRow, 3) = infoFile.Length
        C1FlexGrid1(droppedRow, 4) = infoFile.CreationTime
    End Sub

End Class
